@@ -5,14 +5,16 @@ from pathlib import Path
 import yaml
 import traceback
 import threading
-from typing import List, Dict, Any
+from typing import Any, Callable, Dict, List, Tuple, Union
 from os.path import exists
 
 # External libraries
 import pandas as pd
+from rich.status import Status
 
 # Submodule imports
 from harvest.utils import *
+from harvest.definitions import *
 
 
 class API:
@@ -40,7 +42,7 @@ class API:
     # List of attributes that are required to be in the secret file
     req_keys = []
 
-    def __init__(self, path: str = None):
+    def __init__(self, path: str = None) -> None:
         """
         Performs initializations of the class, such as setting the
         timestamp and loading credentials.
@@ -63,37 +65,23 @@ class API:
             path = "./secret.yaml"
         # Check if file exists. If not, create a secret file
         if not exists(path):
-            config = self.create_secret(path)
+            config = self.create_secret()
         else:
             # Open file
             with open(path, "r") as stream:
                 config = yaml.safe_load(stream)
                 # Check if the file contains all the required parameters
                 if any(key not in config for key in self.req_keys):
-                    config.update(self.create_secret(path))
+                    config.update(self.create_secret())
 
         with open(path, "w") as f:
             yaml.dump(config, f)
 
         self.config = config
 
-    def create_secret(self, path: str):
-        """
-        This method is called when the yaml file with credentials
-        is not found. It returns a dictionary containing the necessary credentials.
-        """
-        # raise Exception(f"{path} was not found.")
-        debugger.warning("Assuming API does not need account information.")
-        return None
-
-    def refresh_cred(self):
-        """
-        Most API endpoints, for security reasons, require a refresh of the access token
-        every now and then. This method should perform a refresh of the access token.
-        """
-        debugger.info(f"Refreshing credentials for {type(self).__name__}.")
-
-    def setup(self, stats: Stats, account: Account, trader_main=None) -> None:
+    def setup(
+        self, stats: Stats, account: Account, trader_main: Callable = None
+    ) -> None:
         """
         This function is called right before the algorithm begins,
         and initializes several runtime parameters like
@@ -132,7 +120,7 @@ class API:
         debugger.debug(f"Poll Interval: {self.poll_interval}")
         debugger.debug(f"{type(self).__name__} setup finished")
 
-    def start(self):
+    def start(self) -> None:
         """
         This method begins streaming data from the API.
 
@@ -145,6 +133,10 @@ class API:
         val, unit = expand_interval(self.poll_interval)
 
         debugger.debug(f"{type(self).__name__} started...")
+        status = Status(
+            f"Waiting for next interval... ({val} {unit})", spinner="material"
+        )
+        status.start()
         if unit == "MIN":
             sleep = val * 60 - 10
             while 1:
@@ -152,7 +144,9 @@ class API:
                 minutes = cur.minute
                 if minutes % val == 0 and minutes != cur_min:
                     self.stats.timestamp = cur
+                    status.stop()
                     self.main()
+                    status.start()
                     time.sleep(sleep)
                 cur_min = minutes
         elif unit == "HR":
@@ -162,7 +156,9 @@ class API:
                 minutes = cur.minute
                 if minutes == 0 and minutes != cur_min:
                     self.stats.timestamp = cur
+                    status.stop()
                     self.main()
+                    status.start()
                     time.sleep(sleep)
                 cur_min = minutes
         else:
@@ -172,11 +168,13 @@ class API:
                 hours = cur.hour
                 if hours == 19 and minutes == 50:
                     self.stats.timestamp = cur
+                    status.stop()
                     self.main()
+                    status.start()
                     time.sleep(80000)
                 cur_min = minutes
 
-    def main(self):
+    def main(self) -> None:
         """
         This method is called at the interval specified by the user.
         It should create a dictionary where each key is the symbol for an asset,
@@ -209,70 +207,38 @@ class API:
 
         self.trader_main(df_dict)
 
-    def exit(self):
+    def exit(self) -> None:
         """
         This function is called after every invocation of algo's handler.
         The intended purpose is for brokers to clear any cache it may have created.
         """
         debugger.debug(f"{type(self).__name__} exited")
 
-    def _exception_handler(func):
+    def create_secret(self) -> Dict[str, str]:
         """
-        Wrapper to handle unexpected errors in the wrapped function.
-        Most functions should be wrapped with this to properly handle errors, such as
-        when internet connection is lost.
-
-        :func: Function to wrap.
-        :returns: The returned value of func if func runs properly. Raises an Exception if func fails.
+        This method is called when the yaml file with credentials
+        is not found. It returns a dictionary containing the necessary credentials.
         """
+        debugger.warning("Assuming API does not need account information.")
 
-        def wrapper(*args, **kwargs):
-            tries = 3
-            while tries > 0:
-                try:
-                    return func(*args, **kwargs)
-                except Exception as e:
-                    self = args[0]
-                    debugger.error(f"Error: {e}")
-                    traceback.print_exc()
-                    debugger.error("Logging out and back in...")
-                    args[0].refresh_cred()
-                    tries -= 1
-                    debugger.error("Retrying...")
-                    continue
-
-        return wrapper
-
-    def _run_once(func):
+    def refresh_cred(self) -> None:
         """
-        Wrapper to only allows wrapped functions to be run once.
-
-        :func: Function to wrap.
-        :returns: The return of the inputted function if it has not been run before and None otherwise.
+        Most API endpoints, for security reasons, require a refresh of the access token
+        every now and then. This method should perform a refresh of the access token.
         """
-
-        ran = False
-
-        def wrapper(*args, **kwargs):
-            nonlocal ran
-            if not ran:
-                ran = True
-                return func(*args, **kwargs)
-            return None
-
-        return wrapper
+        debugger.info(f"Refreshing credentials for {type(self).__name__}.")
 
     # -------------- Streamer methods -------------- #
 
-    def get_current_time(self):
+    def get_current_time(self) -> dt.datetime:
         return now()
 
     def fetch_price_history(
         self,
         symbol: str,
         interval: Interval,
-        start: dt.datetime = None,
-        end: dt.datetime = None,
+        start: Union[str, dt.datetime] = None,
+        end: Union[str, dt.datetime] = None,
     ) -> pd.DataFrame:
         """
         Fetches historical price data for the specified asset and period
@@ -296,7 +262,7 @@ class API:
         price = self.fetch_price_history(symbol, interval, start, end)
         return price[symbol]["close"][-1]
 
-    def fetch_chain_info(self, symbol: str):
+    def fetch_chain_info(self, symbol: str) -> Dict[str, Any]:
         """
         Returns information about the symbol's options
 
@@ -310,7 +276,9 @@ class API:
             f"{type(self).__name__} does not support this streamer method: `fetch_chain_info`."
         )
 
-    def fetch_chain_data(self, symbol: str, date):
+    def fetch_chain_data(
+        self, symbol: str, date: Union[str, dt.datetime]
+    ) -> pd.DataFrame:
         """
         Returns the option chain for the specified symbol.
 
@@ -327,7 +295,7 @@ class API:
             f"{type(self).__name__} does not support this streamer method: `fetch_chain_data`."
         )
 
-    def fetch_option_market_data(self, symbol: str):
+    def fetch_option_market_data(self, symbol: str) -> Dict[str, Any]:
         """
         Retrieves data of specified option.
 
@@ -341,7 +309,7 @@ class API:
             f"{type(self).__name__} does not support this streamer method: `fetch_option_market_data`."
         )
 
-    def fetch_market_hours(self, date: datetime.date):
+    def fetch_market_hours(self, date: dt.date) -> Dict[str, Any]:
         """
         Returns the market hours for a given day.
         Hours are based on the exchange specified in the class's 'exchange' attribute.
@@ -358,7 +326,7 @@ class API:
 
     # ------------- Broker methods ------------- #
 
-    def fetch_stock_positions(self):
+    def fetch_stock_positions(self) -> List[Dict[str, Any]]:
         """
         Returns all current stock positions
 
@@ -372,7 +340,7 @@ class API:
         )
         return []
 
-    def fetch_option_positions(self):
+    def fetch_option_positions(self) -> List[Dict[str, Any]]:
         """
         Returns all current option positions
 
@@ -391,7 +359,7 @@ class API:
         )
         return []
 
-    def fetch_crypto_positions(self):
+    def fetch_crypto_positions(self) -> List[Dict[str, Any]]:
         """
         Returns all current crypto positions
 
@@ -418,7 +386,7 @@ class API:
     #         f"{type(self).__name__} does not support this broker method: `update_option_positions`. Doing nothing."
     #     )
 
-    def fetch_account(self):
+    def fetch_account(self) -> Dict[str, float]:
         """
         Returns current account information from the brokerage.
 
@@ -432,7 +400,7 @@ class API:
             f"{type(self).__name__} does not support this broker method: `fetch_account`."
         )
 
-    def fetch_stock_order_status(self, id):
+    def fetch_stock_order_status(self, id) -> Dict[str, Any]:
         """
         Returns the status of a stock order with the given id.
 
@@ -453,7 +421,7 @@ class API:
             f"{type(self).__name__} does not support this broker method: `fetch_stock_order_status`."
         )
 
-    def fetch_option_order_status(self, id):
+    def fetch_option_order_status(self, id) -> Dict[str, Any]:
         """
         Returns the status of a option order with the given id.
 
@@ -474,7 +442,7 @@ class API:
             f"{type(self).__name__} does not support this broker method: `fetch_option_order_status`."
         )
 
-    def fetch_crypto_order_status(self, id):
+    def fetch_crypto_order_status(self, id) -> Dict[str, Any]:
         """
         Returns the status of a crypto order with the given id.
 
@@ -495,7 +463,7 @@ class API:
             f"{type(self).__name__} does not support this broker method: `fetch_crypto_order_status`."
         )
 
-    def fetch_order_queue(self):
+    def fetch_order_queue(self) -> List[Dict[str, Any]]:
         """
         Returns all current pending orders
 
@@ -540,7 +508,7 @@ class API:
         limit_price: float,
         in_force: str = "gtc",
         extended: bool = False,
-    ):
+    ) -> Dict[str, Any]:
         """
         Places a limit order.
 
@@ -568,7 +536,7 @@ class API:
         limit_price: float,
         in_force: str = "gtc",
         extended: bool = False,
-    ):
+    ) -> Dict[str, Any]:
         """
         Places a limit order.
 
@@ -598,7 +566,7 @@ class API:
         exp_date: dt.datetime,
         strike: float,
         in_force: str = "gtc",
-    ):
+    ) -> Dict[str, Any]:
         """
         Order an option.
 
@@ -620,17 +588,17 @@ class API:
             f"{type(self).__name__} does not support this broker method: `order_option_limit`."
         )
 
-    def cancel_stock_order(self, order_id):
+    def cancel_stock_order(self, order_id) -> None:
         raise NotImplementedError(
             f"{type(self).__name__} does not support this broker method: `cancel_stock_order`."
         )
 
-    def cancel_crypto_order(self, order_id):
+    def cancel_crypto_order(self, order_id) -> None:
         raise NotImplementedError(
             f"{type(self).__name__} does not support this broker method: `cancel_crypto_order`."
         )
 
-    def cancel_option_order(self, order_id):
+    def cancel_option_order(self, order_id) -> None:
         raise NotImplementedError(
             f"{type(self).__name__} does not support this broker method: `cancel_option_order`."
         )
@@ -645,7 +613,7 @@ class API:
         limit_price: float,
         in_force: str = "gtc",
         extended: bool = False,
-    ):
+    ) -> Dict[str, Any]:
         """
         Buys the specified asset.
 
@@ -728,7 +696,7 @@ class API:
         else:
             debugger.error(f"Invalid asset type for {symbol}")
 
-    def cancel(self, order_id):
+    def cancel(self, order_id) -> None:
         for o in self.account.orders.orders:
             if o.order_id == order_id:
                 asset_type = symbol_type(o.symbol)
@@ -820,41 +788,87 @@ class API:
 
     # -------------- Helper methods -------------- #
 
-    def has_interval(self, interval: str):
+    def has_interval(self, interval: Interval) -> bool:
         return interval in self.interval_list
 
     def data_to_occ(
         self, symbol: str, date: dt.datetime, option_type: str, price: float
-    ):
+    ) -> str:
         return data_to_occ(symbol, date, option_type, price)
 
-    def occ_to_data(self, symbol: str):
+    def occ_to_data(self, symbol: str) -> Tuple[str, dt.datetime, str, float]:
         return occ_to_data(symbol)
 
-    def current_timestamp(self):
-        return self.timestamp
+    def current_timestamp(self) -> dt.datetime:
+        return now()
+
+    def _exception_handler(func: Callable) -> Callable:
+        """
+        Wrapper to handle unexpected errors in the wrapped function.
+        Most functions should be wrapped with this to properly handle errors, such as
+        when internet connection is lost.
+
+        :func: Function to wrap.
+        :returns: The returned value of func if func runs properly. Raises an Exception if func fails.
+        """
+
+        def wrapper(*args, **kwargs):
+            tries = 3
+            while tries > 0:
+                try:
+                    return func(*args, **kwargs)
+                except Exception as e:
+                    self = args[0]
+                    debugger.error(f"Error: {e}")
+                    traceback.print_exc()
+                    debugger.error("Logging out and back in...")
+                    args[0].refresh_cred()
+                    tries -= 1
+                    debugger.error("Retrying...")
+                    continue
+
+        return wrapper
+
+    def _run_once(func: Callable) -> Callable:
+        """
+        Wrapper to only allows wrapped functions to be run once.
+
+        :func: Function to wrap.
+        :returns: The return of the inputted function if it has not been run before and None otherwise.
+        """
+
+        ran = False
+
+        def wrapper(*args, **kwargs):
+            nonlocal ran
+            if not ran:
+                ran = True
+                return func(*args, **kwargs)
+            return None
+
+        return wrapper
+
+    def _validate_order(self, side: str, quantity: float, limit_price: float) -> None:
+        assert side in ("buy", "sell"), "Side must be either 'buy' or 'sell'"
+        assert quantity >= 0, "Quantity must be nonnegative"
+        assert limit_price >= 0, "Limit price must be nonnegative"
 
 
 class StreamAPI(API):
     """ """
 
-    def __init__(self, path: str = None):
+    def __init__(self, path: str = None) -> None:
         super().__init__(path)
 
-        self.block_lock = (
-            threading.Lock()
-        )  # Lock for streams that receive data asynchronously.
+        # Lock for streams that receive data asynchronously.
+        self.block_lock = threading.Lock()
         self.block_queue = {}
         self.first = True
 
-    def setup(self, stats, account, trader_main=None) -> None:
-        super().setup(stats, account, trader_main)
-        self.blocker = {}
-
-    def start(self):
+    def start(self) -> None:
         debugger.debug(f"{type(self).__name__} started...")
 
-    def main(self, df_dict):
+    def main(self, df_dict: Dict[str, Any]) -> None:
         """
         Streaming is event driven, so sometimes not all data comes in at once.
         StreamAPI class
@@ -900,14 +914,14 @@ class StreamAPI(API):
         self.got = got
         self.block_lock.release()
 
-    def timeout(self):
+    def timeout(self) -> None:
         debugger.debug("Begin timeout timer")
         time.sleep(1)
         if not self.all_recv:
             debugger.debug("Force flush")
             self.flush()
 
-    def flush(self):
+    def flush(self) -> None:
         # For missing data, return a OHLC with all zeroes.
         self.block_lock.acquire()
         for n in self.needed:
