@@ -1,5 +1,6 @@
 # Builtins
 import yaml
+import asyncio
 import threading
 import datetime as dt
 from typing import Any, Dict, List, Tuple
@@ -7,11 +8,13 @@ from typing import Any, Dict, List, Tuple
 # External libraries
 import pandas as pd
 from alpaca_trade_api.rest import REST, TimeFrame, URL
+from alpaca_trade_api.entity import Bar
 from alpaca_trade_api import Stream
 
 # Submodule imports
 from harvest.api._base import StreamAPI, API
 from harvest.utils import *
+from harvest.definitions import *
 
 
 class Alpaca(StreamAPI):
@@ -26,7 +29,7 @@ class Alpaca(StreamAPI):
         path: str = None,
         is_basic_account: bool = False,
         paper_trader: bool = False,
-    ):
+    ) -> None:
         super().__init__(path)
 
         if self.config is None:
@@ -55,37 +58,9 @@ class Alpaca(StreamAPI):
         self.data_lock = threading.Lock()
         self.data = {}
 
-    async def update_data(self, bar):
-        # Update data with the latest bars
-        self.data_lock.acquire()
-        bar = bar.__dict__["_raw"]
-        symbol = bar["symbol"]
-        df = pd.DataFrame(
-            [
-                {
-                    "t": bar["timestamp"],
-                    "o": bar["open"],
-                    "h": bar["high"],
-                    "l": bar["low"],
-                    "c": bar["close"],
-                    "v": bar["volume"],
-                }
-            ]
-        )
-
-        symbol = f"@{symbol}" if is_crypto(symbol) else symbol
-        debugger.info(f"Got data for {symbol}")
-        df = self._format_df(df, symbol)
-        self.data[symbol] = df
-        if set(self.data.keys()) == set(self.watch_stock + self.watch_crypto):
-            data = self.data
-            self.data = {}
-            self.data_lock.release()
-            self.trader_main(data)
-        else:
-            self.data_lock.release()
-
-    def setup(self, stats, account, trader_main=None):
+    def setup(
+        self, stats: Stats, account: Account, trader_main: Callable = None
+    ) -> None:
         super().setup(stats, account, trader_main)
 
         self.watch_stock = []
@@ -99,29 +74,41 @@ class Alpaca(StreamAPI):
             else:
                 self.watch_stock.append(s)
 
-        self.stream.on_bar(*(self.watch_stock + cryptos))(self.update_data)
+        asyncio.set_event_loop(asyncio.new_event_loop())
+        self.stream.on_bar(*(self.watch_stock + cryptos))(self._update_data)
 
         self.option_cache = {}
 
-    def start(self):
+    def start(self) -> None:
         self.stream.run()
 
-    def exit(self):
+    def exit(self) -> None:
         self.option_cache = {}
 
     # -------------- Streamer methods -------------- #
 
+    @API._exception_handler
+    def get_current_time(self) -> dt.datetime:
+        ret = self.api.get_clock().__dict__["_raw"]
+        # Convert to ISO 8601 by removing nanoseconds
+        index = ret["timestamp"].index(".")
+        timestamp = ret["timestamp"][:index] + ret["timestamp"][index + 9 :]
+        return dt.datetime.fromisoformat(timestamp)
+
     @API._exception_handler
     def fetch_price_history(
         self,
         symbol: str,
         interval: Interval,
-        start: dt.datetime = None,
-        end: dt.datetime = None,
-    ):
+        start: Union[str, dt.datetime] = None,
+        end: Union[str, dt.datetime] = None,
+    ) -> pd.DataFrame:
 
         debugger.debug(f"Fetching {symbol} {interval} price history")
 
+        start = convert_input_to_datetime(start)
+        end = convert_input_to_datetime(end)
+
         if start is None:
             start = now() - dt.timedelta(days=365 * 5)
         if end is None:
@@ -130,31 +117,34 @@ class Alpaca(StreamAPI):
         if start >= end:
             return pd.DataFrame()
 
-        df = self.get_data_from_alpaca(symbol, interval, start, end)
-        return df
+        return self._get_data_from_alpaca(symbol, interval, start, end)
 
     @API._exception_handler
-    def fetch_chain_info(self, symbol: str):
+    def fetch_chain_info(self, symbol: str) -> None:
         raise NotImplementedError("Alpaca does not support options.")
 
     @API._exception_handler
-    def fetch_chain_data(self, symbol: str, date: dt.datetime):
+    def fetch_chain_data(self, symbol: str, date: dt.datetime) -> None:
         raise NotImplementedError("Alpaca does not support options.")
 
     @API._exception_handler
-    def fetch_option_market_data(self, occ_symbol: str):
+    def fetch_option_market_data(self, occ_symbol: str) -> None:
         raise NotImplementedError("Alpaca does not support options.")
 
     @API._exception_handler
-    def fetch_market_hours(self, date: datetime.date):
-        ret = self.api.get_clock()
-        return ret.__dict__["_raw"]
+    def fetch_market_hours(self, date: datetime.date) -> Dict[str, Any]:
+        ret = self.api.get_clock().__dict__["_raw"]
+        return {
+            "is_open": ret["is_open"],
+            "open_at": dt.datetime.fromisoformat(ret["next_open"]),
+            "close_at": dt.datetime.fromisoformat(ret["next_close"]),
+        }
 
     # ------------- Broker methods ------------- #
 
     @API._exception_handler
-    def fetch_stock_positions(self):
-        def fmt(stock: Dict[str, Any]):
+    def fetch_stock_positions(self) -> List[Dict[str, Any]]:
+        def fmt(stock: Dict[str, Any]) -> Dict[str, Any]:
             return {
                 "symbol": stock["symbol"],
                 "avg_price": float(stock["avg_entry_price"]),
@@ -169,19 +159,19 @@ class Alpaca(StreamAPI):
         ]
 
     @API._exception_handler
-    def fetch_option_positions(self):
+    def fetch_option_positions(self) -> List:
         debugger.error("Alpaca does not support options. Returning an empty list.")
         return []
 
     @API._exception_handler
-    def fetch_crypto_positions(self, key=None):
+    def fetch_crypto_positions(self) -> List:
         if self.basic:
             debugger.error(
                 "Alpaca basic accounts do not support crypto. Returning an empty list."
             )
             return []
 
-        def fmt(crypto: Dict[str, Any]):
+        def fmt(crypto: Dict[str, Any]) -> Dict[str, Any]:
             return {
                 "symbol": "@" + crypto["symbol"],
                 "avg_price": float(crypto["avg_entry_price"]),
@@ -196,11 +186,11 @@ class Alpaca(StreamAPI):
         ]
 
     @API._exception_handler
-    def update_option_positions(self, positions: List[Any]):
+    def update_option_positions(self, positions: List[Any]) -> None:
         debugger.error("Alpaca does not support options. Doing nothing.")
 
     @API._exception_handler
-    def fetch_account(self):
+    def fetch_account(self) -> Dict[str, Any]:
         account = self.api.get_account().__dict__["_raw"]
         return {
             "equity": float(account["equity"]),
@@ -211,23 +201,23 @@ class Alpaca(StreamAPI):
         }
 
     @API._exception_handler
-    def fetch_stock_order_status(self, id: str):
-        return self.api.get_order(id).__dict__["_raw"]
+    def fetch_stock_order_status(self, order_id: str) -> Dict[str, Any]:
+        return self.api.get_order(order_id).__dict__["_raw"]
 
     @API._exception_handler
-    def fetch_option_order_status(self, id):
+    def fetch_option_order_status(self, order_id: str) -> None:
         raise NotImplementedError("Alpaca does not support options.")
 
     @API._exception_handler
-    def fetch_crypto_order_status(self, id):
+    def fetch_crypto_order_status(self, order_id: str) -> Dict[str, Any]:
         if self.basic:
             raise Exception("Alpaca basic accounts do not support crypto.")
-        return self.api.get_order(id).__dict__["_raw"]
+        return self.api.get_order(order_id).__dict__["_raw"]
 
     @API._exception_handler
-    def fetch_order_queue(self):
+    def fetch_order_queue(self) -> List[Dict[str, Any]]:
         return [
-            self.format_order_status(pos.__dict__["_raw"])
+            self._format_order_status(pos.__dict__["_raw"])
             for pos in self.api.list_orders()
         ]
 
@@ -241,7 +231,9 @@ class Alpaca(StreamAPI):
         limit_price: float,
         in_force: str = "gtc",
         extended: bool = False,
-    ):
+    ) -> Dict[str, Any]:
+
+        self._validate_order(side, quantity, limit_price)
 
         order = self.api.submit_order(
             symbol,
@@ -254,12 +246,25 @@ class Alpaca(StreamAPI):
         ).__dict__["_raw"]
 
         return {
-            "type": "CRYPTO",
+            "type": "STOCK",
             "id": order["id"],
             "symbol": symbol,
             "alpaca": order,
         }
 
+    def order_option_limit(
+        self,
+        side: str,
+        symbol: str,
+        quantity: int,
+        limit_price: float,
+        option_type,
+        exp_date: dt.datetime,
+        strike,
+        in_force: str = "gtc",
+    ) -> None:
+        raise NotImplementedError("Alpaca does not support options.")
+
     def order_crypto_limit(
         self,
         side: str,
@@ -268,10 +273,12 @@ class Alpaca(StreamAPI):
         limit_price: float,
         in_force: str = "gtc",
         extended: bool = False,
-    ):
+    ) -> Dict[str, Any]:
         if self.basic:
             raise Exception("Alpaca basic accounts do not support crypto.")
 
+        self._validate_order(side, quantity, limit_price)
+
         symbol = symbol[1:]
 
         order = self.api.submit_order(
@@ -291,35 +298,66 @@ class Alpaca(StreamAPI):
             "alpaca": order,
         }
 
-    def order_option_limit(
-        self,
-        side: str,
-        symbol: str,
-        quantity: int,
-        limit_price: float,
-        option_type,
-        exp_date: dt.datetime,
-        strike,
-        in_force: str = "gtc",
-    ):
+    def cancel_stock_order(self, order_id: str) -> Dict[str, Any]:
+        ret = self.api.cancel_order(order_id)
+        return {"id": order_id}
+
+    def cancel_option_order(self, order_id: str) -> None:
         raise NotImplementedError("Alpaca does not support options.")
 
-    def cancel_stock_order(self, order_id):
-        ret = self.api.cancel_order(order_id)
-        return ret.__dict__["_raw"]
-
-    def cancel_crypto_order(self, order_id):
+    def cancel_crypto_order(self, order_id: str) -> Dict[str, Any]:
         if self.basic:
             raise Exception("Alpaca basic accounts do not support crypto.")
         ret = self.api.cancel_order(order_id)
-        return ret.__dict__["_raw"]
-
-    def cancel_option_order(self, order_id):
-        raise NotImplementedError("Alpaca does not support options.")
+        return {"id": order_id}
 
     # ------------- Helper methods ------------- #
 
-    def format_order_status(self, order: Dict[str, Any], is_stock: bool = True):
+    def create_secret(self) -> Dict[str, str]:
+        import harvest.wizard as wizard
+
+        w = wizard.Wizard()
+
+        w.println("Hmm, looks like you haven't set up an api key for Alpaca.")
+        should_setup = w.get_bool("Do you want to set it up now?", default="y")
+
+        if not should_setup:
+            w.println("You can't use Alpaca without an API key.")
+            w.println(
+                "You can set up the credentials manually, or use other streamers."
+            )
+            return False
+
+        w.println("Alright! Let's get started")
+
+        have_account = w.get_bool("Do you have an Alpaca account?", default="y")
+        if not have_account:
+            w.println(
+                "In that case you'll first need to make an account. This takes a few steps."
+            )
+            w.println(
+                "First visit: https://alpaca.markets/ and sign up. Hit Enter or Return for the next step."
+            )
+            w.wait_for_input()
+            w.println("Follow the setups to make an individual or buisness account.")
+            w.wait_for_input()
+            w.println(
+                "In the sidebar of the Alpaca Dashboard, note if you want to use a live account (real money) or a paper account (simulated)."
+            )
+            w.wait_for_input()
+            w.println(
+                "On the right-hand side, in the Your API Keys box, click View and then Generate API Key. Copy these somewhere safe."
+            )
+            w.wait_for_input()
+
+        api_key_id = w.get_string("Enter your API key ID")
+        secret_key = w.get_password("Enter your API secret key")
+
+        return {"alpaca_api_key": f"{api_key_id}", "alpaca_secret_key": f"{secret_key}"}
+
+    def _format_order_status(
+        self, order: Dict[str, Any], is_stock: bool = True
+    ) -> Dict[str, Any]:
         return {
             "type": "STOCK" if is_stock else "CRYPTO",
             "id": order["id"],
@@ -332,7 +370,7 @@ class Alpaca(StreamAPI):
             "alpaca": order,
         }
 
-    def get_data_from_alpaca(
+    def _get_data_from_alpaca(
         self,
         symbol: str,
         interval: Interval,
@@ -378,7 +416,7 @@ class Alpaca(StreamAPI):
         df = aggregate_df(df, interval)
         return df
 
-    def _format_df(self, df: pd.DataFrame, symbol: str):
+    def _format_df(self, df: pd.DataFrame, symbol: str) -> pd.DataFrame:
         df = df[["t", "o", "h", "l", "c", "v"]]
         df.rename(
             columns={
@@ -392,53 +430,39 @@ class Alpaca(StreamAPI):
             inplace=True,
         )
 
-        df["timestamp"] = pd.to_datetime(df["timestamp"], utc=True)
-        df.set_index("timestamp", inplace=True)
-
+        df.index = pd.DatetimeIndex(
+            pd.to_datetime(df["timestamp"], utc=True), tz=dt.timezone.utc
+        )
+        df.drop(columns=["timestamp"], inplace=True)
         df.columns = pd.MultiIndex.from_product([[symbol], df.columns])
-
         return df.dropna()
 
-    def create_secret(self, path: str) -> bool:
-        import harvest.wizard as wizard
+    async def _update_data(self, bar: Bar) -> None:
+        # Update data with the latest bars
+        self.data_lock.acquire()
+        bar = bar.__dict__["_raw"]
+        symbol = bar["symbol"]
+        df = pd.DataFrame(
+            [
+                {
+                    "t": bar["timestamp"],
+                    "o": bar["open"],
+                    "h": bar["high"],
+                    "l": bar["low"],
+                    "c": bar["close"],
+                    "v": bar["volume"],
+                }
+            ]
+        )
 
-        w = wizard.Wizard()
-
-        w.println("Hmm, looks like you haven't set up an api key for Alpaca.")
-        should_setup = w.get_bool("Do you want to set it up now?", default="y")
-
-        if not should_setup:
-            w.println("You can't use Alpaca without an API key.")
-            w.println(
-                "You can set up the credentials manually, or use other streamers."
-            )
-            return False
-
-        w.println("Alright! Let's get started")
-
-        have_account = w.get_bool("Do you have an Alpaca account?", default="y")
-        if not have_account:
-            w.println(
-                "In that case you'll first need to make an account. This takes a few steps."
-            )
-            w.println(
-                "First visit: https://alpaca.markets/ and sign up. Hit Enter or Return for the next step."
-            )
-            w.wait_for_input()
-            w.println("Follow the setups to make an individual or buisness account.")
-            w.wait_for_input()
-            w.println(
-                "In the sidebar of the Alpaca Dashboard, note if you want to use a live account (real money) or a paper account (simulated)."
-            )
-            w.wait_for_input()
-            w.println(
-                "On the right-hand side, in the Your API Keys box, click View and then Generate API Key. Copy these somewhere safe."
-            )
-            w.wait_for_input()
-
-        api_key_id = w.get_string("Enter your API key ID")
-        secret_key = w.get_password("Enter your API secret key")
-
-        w.println(f"All steps are complete now 🎉. Generating {path}...")
-
-        return {"alpaca_api_key": f"{api_key_id}", "alpaca_secret_key": f"{secret_key}"}
+        symbol = f"@{symbol}" if is_crypto(symbol) else symbol
+        debugger.info(f"Got data for {symbol}")
+        df = self._format_df(df, symbol)
+        self.data[symbol] = df
+        if set(self.data.keys()) == set(self.watch_stock + self.watch_crypto):
+            data = self.data
+            self.data = {}
+            self.data_lock.release()
+            self.trader_main(data)
+        else:
+            self.data_lock.release()
