@@ -10,6 +10,7 @@ import pandas as pd
 # Submodule imports
 from harvest.api._base import API
 from harvest.utils import *
+from harvest.definitions import *
 
 
 class Kraken(API):
@@ -112,7 +113,7 @@ class Kraken(API):
         v: k for k, v in crypto_ticker_to_kraken_names.items()
     }
 
-    def __init__(self, path: str = None):
+    def __init__(self, path: str = None) -> None:
         super().__init__(path)
 
         if self.config is None:
@@ -124,7 +125,9 @@ class Kraken(API):
             self.config["kraken_api_key"], self.config["kraken_secret_key"]
         )
 
-    def setup(self, stats, account, trader_main=None):
+    def setup(
+        self, stats: Stats, account: Account, trader_main: Callable = None
+    ) -> None:
         super().setup(stats, account, trader_main)
         self.watch_crypto = []
         for sym in self.stats.watchlist_cfg:
@@ -135,48 +138,40 @@ class Kraken(API):
 
         self.option_cache = {}
 
-    def exit(self):
-        self.option_cache = {}
-
-    def main(self):
+    def main(self) -> None:
         df_dict = {}
-        df_dict.update(self.fetch_latest_crypto_price())
+        df_dict.update(self._fetch_latest_crypto_price())
 
         self.trader_main(df_dict)
 
-    @API._exception_handler
-    def fetch_latest_crypto_price(self):
-        dfs = {}
-        for symbol in self.watch_crypto:
-            dfs[symbol] = self.fetch_price_history(
-                symbol,
-                self.interval[symbol]["interval"],
-                now() - dt.timedelta(days=7),
-                now(),
-            ).iloc[[-1]]
-        return dfs
+    def exit(self) -> None:
+        self.option_cache = {}
 
     # -------------- Streamer methods -------------- #
 
+    @API._exception_handler
+    def get_current_time(self) -> dt.datetime:
+        ret = self._get_result(self.api.query_public("Time"))
+        return dt.datetime.fromtimestamp(ret["unixtime"], dt.timezone.utc)
+
     @API._exception_handler
     def fetch_price_history(
         self,
         symbol: str,
         interval: Interval,
-        start: dt.datetime = None,
-        end: dt.datetime = None,
-    ):
+        start: Union[str, dt.datetime] = None,
+        end: Union[str, dt.datetime] = None,
+    ) -> pd.DataFrame:
 
         debugger.debug(f"Fetching {symbol} {interval} price history")
 
+        start = convert_input_to_datetime(start)
+        end = convert_input_to_datetime(end)
+
         if start is None:
-            start = now() - dt.timedelta(days=365 * 5)
-        elif not has_timezone(start):
-            start = set_system_timezone(start)
+            start = now() - dt.timedelta(hours=12)
         if end is None:
             end = now()
-        elif not has_timezone(end):
-            end = set_system_timezone(end)
 
         if start >= end:
             return pd.DataFrame()
@@ -186,23 +181,24 @@ class Kraken(API):
                 f"Interval {interval} not in interval list. Possible options are: {self.interval_list}"
             )
         val, unit = expand_interval(interval)
-        df = self.get_data_from_kraken(symbol, val, unit, start, end)
+        df = self._get_data_from_kraken(symbol, val, unit, start, end)
 
         return df
 
-    def fetch_chain_info(self, symbol: str):
+    def fetch_chain_info(self, symbol: str) -> None:
         raise NotImplementedError("Kraken does not support options.")
 
-    def fetch_chain_data(self, symbol: str, date: dt.datetime):
+    def fetch_chain_data(self, symbol: str, date: dt.datetime) -> None:
         raise NotImplementedError("Kraken does not support options.")
 
-    def fetch_option_market_data(self, occ_symbol: str):
+    def fetch_option_market_data(self, occ_symbol: str) -> None:
         raise NotImplementedError("Kraken does not support options.")
 
-    def fetch_market_hours(self, date: datetime.date):
+    def fetch_market_hours(self, date: datetime.date) -> None:
         # Crypto markets are always open.
+        ret = self._get_result(self.api.query_public("SystemStatus"))
         return {
-            "is_open": True,
+            "is_open": ret["status"] == "online",
             "open_at": None,
             "close_at": None,
         }
@@ -210,18 +206,18 @@ class Kraken(API):
     # ------------- Broker methods ------------- #
 
     @API._exception_handler
-    def fetch_stock_positions(self):
+    def fetch_stock_positions(self) -> List:
         debugger.error("Kraken does not support stocks. Returning an empty list.")
         return []
 
     @API._exception_handler
-    def fetch_option_positions(self):
+    def fetch_option_positions(self) -> List:
         debugger.error("Kraken does not support options")
         return []
 
     @API._exception_handler
-    def fetch_crypto_positions(self):
-        positions = self.get_result(self.api.query_private("OpenPositions"))
+    def fetch_crypto_positions(self) -> List[Dict[str, Any]]:
+        positions = self._get_result(self.api.query_private("OpenPositions"))
 
         def fmt(crypto: Dict[str, Any]):
             # Remove the currency
@@ -241,8 +237,8 @@ class Kraken(API):
         debugger.error("Kraken does not support options. Doing nothing.")
 
     @API._exception_handler
-    def fetch_account(self):
-        account = self.get_result(self.api.query_private("Balance"))
+    def fetch_account(self) -> Dict[str, Any]:
+        account = self._get_result(self.api.query_private("Balance"))
         if account is None:
             equity = 0
             cash = 0
@@ -257,14 +253,14 @@ class Kraken(API):
             "kraken": account,
         }
 
-    def fetch_stock_order_status(self, order_id: str):
+    def fetch_stock_order_status(self, order_id: str) -> None:
         return NotImplementedError("Kraken does not support stocks.")
 
-    def fetch_option_order_status(self, order_id: str):
+    def fetch_option_order_status(self, order_id: str) -> None:
         raise Exception("Kraken does not support options.")
 
     @API._exception_handler
-    def fetch_crypto_order_status(self, order_id: str):
+    def fetch_crypto_order_status(self, order_id: str) -> Dict[str, Any]:
         order = self.api.query_private("QueryOrders", {"txid": order_id})
         symbol = kraken_names_to_crypto_ticker.get(crypto["descr"]["pair"][:-4])
         return {
@@ -282,8 +278,8 @@ class Kraken(API):
     # --------------- Methods for Trading --------------- #
 
     @API._exception_handler
-    def fetch_order_queue(self):
-        open_orders = self.get_result(self.api.query_private("OpenOrders"))
+    def fetch_order_queue(self) -> Dict[str, Any]:
+        open_orders = self._get_result(self.api.query_private("OpenOrders"))
         open_orders = open_orders["open"]
 
         def fmt(crypto: Dict[str, Any]):
@@ -310,15 +306,20 @@ class Kraken(API):
         limit_price: float,
         in_force: str = "gtc",
         extended: bool = False,
-    ):
-        order = self.get_result(
+    ) -> Dict[str, Any]:
+
+        self._validate_order(side, quantity, limit_price)
+
+        kraken_symbol = self._ticker_to_kraken(symbol)
+        order = self._get_result(
             self.api.query_private(
                 "AddOrder",
                 {
                     "ordertype": "limit",
                     "type": side,
                     "volume": quantity,
-                    "pair": symbol,
+                    "price": limit_price,
+                    "pair": "XXBTZUSD",  # symbol,
                 },
             )
         )
@@ -330,85 +331,12 @@ class Kraken(API):
             "kraken": order,
         }
 
-    def cancel_crypto_order(self, order_id):
+    def cancel_crypto_order(self, order_id) -> None:
         self.api.query_private("CancelOrder", {"txid": order_id})
 
     # ------------- Helper methods ------------- #
 
-    def get_data_from_kraken(
-        self,
-        symbol: str,
-        multiplier: int,
-        timespan: str,
-        start: dt.datetime,
-        end: dt.datetime,
-    ) -> pd.DataFrame:
-        if timespan == "MIN":
-            multiplier *= 1
-        elif timespan == "HR":
-            multiplier *= 60
-        elif timespan == "DAY":
-            multiplier *= 1440
-
-        if is_crypto(symbol):
-            temp_symbol = self.ticker_to_kraken(symbol)
-        else:
-            raise Exception("Kraken does not support stocks.")
-        bars = self.get_result(
-            self.api.query_public(
-                "OHLC",
-                {"pair": temp_symbol, "interval": multiplier, "since": end.timestamp},
-            )
-        )
-        df = pd.DataFrame(
-            bars[temp_symbol],
-            columns=[
-                "timestamp",
-                "open",
-                "high",
-                "low",
-                "close",
-                "vwap",
-                "volume",
-                "count",
-            ],
-        )
-        df = self._format_df(df, symbol)
-        df = df.loc[start:end]
-        return df
-
-    def _format_df(self, df: pd.DataFrame, symbol: str):
-        df = df[["timestamp", "open", "high", "low", "close", "volume"]].astype(float)
-        df.index = pd.to_datetime(df["timestamp"], unit="s", utc=True)
-        df = df.drop(columns=["timestamp"])
-        df.columns = pd.MultiIndex.from_product([[symbol], df.columns])
-
-        return df.dropna()
-
-    def ticker_to_kraken(self, ticker: str):
-        if not is_crypto(ticker):
-            raise Exception("Kraken does not support stocks.")
-
-        if ticker[1:] in self.crypto_ticker_to_kraken_names:
-            # Currently Harvest supports trades for USD and not other currencies.
-            kraken_ticker = self.crypto_ticker_to_kraken_names.get(ticker[1:]) + "USD"
-            asset_pairs = self.get_result(self.api.query_public("AssetPairs")).keys()
-            if kraken_ticker in asset_pairs:
-                return kraken_ticker
-            else:
-                raise Exception(f"{kraken_ticker} is not a valid asset pair.")
-        else:
-            raise Exception(f"Kraken does not support ticker {ticker}.")
-
-    def get_result(self, response: Dict[str, Any]):
-        """Given a kraken response from an endpoint, either raise an error if an
-        error exists or return the data in the results key.
-        """
-        if len(response["error"]) > 0:
-            raise Exception("\n".join(response["error"]))
-        return response.get("result", None)
-
-    def create_secret(self, path: str) -> bool:
+    def create_secret(self) -> Dict[str, str]:
         import harvest.wizard as wizard
 
         w = wizard.Wizard()
@@ -442,6 +370,94 @@ class Kraken(API):
         api_key_id = w.get_string("Enter your API key ID")
         secret_key = w.get_password("Enter your API secret key")
 
-        w.println(f"All steps are complete now 🎉. Generating {path}...")
-
         return {"kraken_api_key": f"{api_key_id}", "kraken_secret_key": f"{secret_key}"}
+
+    def _get_data_from_kraken(
+        self,
+        symbol: str,
+        multiplier: int,
+        timespan: str,
+        start: dt.datetime,
+        end: dt.datetime,
+    ) -> pd.DataFrame:
+        if timespan == "MIN":
+            multiplier *= 1
+        elif timespan == "HR":
+            multiplier *= 60
+        elif timespan == "DAY":
+            multiplier *= 1440
+
+        if is_crypto(symbol):
+            temp_symbol = self._ticker_to_kraken(symbol)
+        else:
+            raise Exception("Kraken does not support stocks.")
+        bars = self._get_result(
+            self.api.query_public(
+                "OHLC",
+                {"pair": temp_symbol, "interval": multiplier, "since": 0},
+            )
+        )
+
+        df = pd.DataFrame(
+            bars[temp_symbol],
+            columns=[
+                "timestamp",
+                "open",
+                "high",
+                "low",
+                "close",
+                "vwap",
+                "volume",
+                "count",
+            ],
+        )
+
+        df = self._format_df(df, symbol)
+        df = df.loc[start:end]
+        return df
+
+    def _format_df(self, df: pd.DataFrame, symbol: str) -> pd.DataFrame:
+        df = df[["timestamp", "open", "high", "low", "close", "volume"]].astype(float)
+        df.index = pd.DatetimeIndex(
+            pd.to_datetime(df["timestamp"].astype(int), unit="s", utc=True),
+            tz=dt.timezone.utc,
+        )
+        df = df.drop(columns=["timestamp"])
+        df.columns = pd.MultiIndex.from_product([[symbol], df.columns])
+
+        return df.dropna()
+
+    def _ticker_to_kraken(self, ticker: str) -> str:
+        if not is_crypto(ticker):
+            raise Exception("Kraken does not support stocks.")
+
+        if ticker[1:] in self.crypto_ticker_to_kraken_names:
+            # Currently Harvest supports trades for USD and not other currencies.
+            kraken_ticker = self.crypto_ticker_to_kraken_names.get(ticker[1:]) + "USD"
+            asset_pairs = self._get_result(self.api.query_public("AssetPairs")).keys()
+            if kraken_ticker in asset_pairs:
+                return kraken_ticker
+            else:
+                raise Exception(f"{kraken_ticker} is not a valid asset pair.")
+        else:
+            raise Exception(f"Kraken does not support ticker {ticker}.")
+
+    def _get_result(self, response: Dict[str, Any]) -> Dict[str, Any]:
+        """Given a kraken response from an endpoint, either raise an error if an
+        error exists or return the data in the results key.
+        """
+        if len(response["error"]) > 0:
+            raise Exception("\n".join(response["error"]))
+        return response.get("result", None)
+
+    @API._exception_handler
+    def _fetch_latest_crypto_price(self) -> Dict[str, pd.DataFrame]:
+        dfs = {}
+        for symbol in self.watch_crypto:
+            dfs[symbol] = self.fetch_price_history(
+                symbol,
+                self.interval[symbol]["interval"],
+                now() - dt.timedelta(days=7),
+                now(),
+            ).iloc[[-1]]
+        return dfs
