@@ -14,6 +14,8 @@ import yaml
 from harvest.api._base import API
 from harvest.api.dummy import DummyStreamer
 from harvest.utils import *
+from harvest.definitions import *
+from harvest.storage import BaseStorage
 
 
 class PaperBroker(API):
@@ -30,10 +32,17 @@ class PaperBroker(API):
         Interval.HR_1,
         Interval.DAY_1,
     ]
-    req_keys = []
 
-    def __init__(self, path: str = None, streamer=None, commission_fee=0, save=False):
+    def __init__(
+        self,
+        path: str = None,
+        streamer: API = None,
+        commission_fee: Union[float, str, Dict[str, Any]] = 0,
+        save: bool = False,
+    ) -> None:
         """
+        :path: Path to a configuration file holding account information for the user.
+        :streamer: A streamer to get asset prices and the current time.
         :commission_fee: When this is a number it is assumed to be a flat price
             on all buys and sells of assets. When this is a string formatted as
             'XX%' then it is assumed that commission fees are that percent of the
@@ -53,7 +62,7 @@ class PaperBroker(API):
         self.commission_fee = commission_fee
         self.save = save
 
-        self.streamer = DummyStreamer() if streamer is None else streamer
+        self.streamer = None
 
         if path is None:
             self.save_path = "./save"
@@ -80,7 +89,9 @@ class PaperBroker(API):
         self.buying_power = self.config["paper_buying_power"]
         self.multiplier = self.config["paper_multiplier"]
 
-    def _load_account(self):
+        debugger.debug("Broker state: {}".format(self.config))
+
+    def _load_account(self) -> None:
         with open(self.save_path, "rb") as stream:
             save_data = pickle.load(stream)
 
@@ -99,7 +110,7 @@ class PaperBroker(API):
             self.orders = orders["orders"]
             self.order_id = orders["order_id"]
 
-    def _save_account(self):
+    def _save_account(self) -> None:
         with open(self.save_path, "wb") as stream:
             save_data = {
                 "account": {
@@ -120,15 +131,22 @@ class PaperBroker(API):
             }
             pickle.dump(save_data, stream)
 
-    def _delete_account(self):
+    def _delete_account(self) -> None:
         try:
             os.remove(self.save_path)
             debugger.debug("Removed saved account file.")
         except:
             debugger.warning("Saved account file does not exists.")
 
-    def setup(self, stats, account, trader_main=None):
+    def setup(
+        self, stats: Stats, account: Account, trader_main: Callable = None
+    ) -> None:
         super().setup(stats, account, trader_main)
+        self.backtest = False
+
+    def setup_backtest(self, storage: BaseStorage) -> None:
+        self.backtest = True
+        self.storage = storage
 
     # -------------- Streamer methods -------------- #
 
@@ -161,6 +179,7 @@ class PaperBroker(API):
     def fetch_account(self) -> Dict[str, Any]:
         self.equity = self._calc_equity()
         self._save_account()
+
         return {
             "equity": self.equity,
             "cash": self.cash,
@@ -172,12 +191,21 @@ class PaperBroker(API):
         ret = next(r for r in self.orders if r["order_id"] == order_id)
         sym = ret["symbol"]
 
-        price = self.streamer.fetch_price_history(
-            sym,
-            self.stats.watchlist_cfg[sym]["interval"],
-            self.streamer.get_current_time() - dt.timedelta(days=7),
-            self.streamer.get_current_time(),
-        )[sym]["close"][-1]
+        debugger.debug(f"Backtest: {self.backtest}")
+
+        if self.backtest:
+            price = self.storage.load(sym, self.stats.watchlist_cfg[sym]["interval"])[
+                sym
+            ]["close"][-1]
+        else:
+            price = self.streamer.fetch_price_history(
+                sym,
+                self.stats.watchlist_cfg[sym]["interval"],
+                self.streamer.get_current_time() - dt.timedelta(days=7),
+                self.streamer.get_current_time(),
+            )[sym]["close"][-1]
+
+        debugger.debug(f"Price of {sym} is {price}")
 
         qty = ret["quantity"]
         original_price = price * qty
@@ -187,10 +215,12 @@ class PaperBroker(API):
             pos = next((r for r in lst if r["symbol"] == sym), None)
             if ret["side"] == "buy":
                 # Check to see if user has enough funds to buy the stock
+                debugger.debug(f"Original price: {original_price}")
                 actual_price = self.apply_commission(
-                    original_price, self.commission_fee, "sell"
+                    original_price, self.commission_fee, "buy"
                 )
-                if self.buying_power < actual_price:
+                # Check if user has enough buying power
+                if self.buying_power + ret["limit_price"] * qty < actual_price:
                     debugger.error(
                         f"""Not enough buying power.\n Total price ({actual_price}) exceeds buying power {self.buying_power}.\n Reduce purchase quantity or increase buying power."""
                     )
@@ -210,6 +240,7 @@ class PaperBroker(API):
                         pos["quantity"] = pos["quantity"] + qty
 
                     self.cash -= actual_price
+                    self.buying_power += ret["limit_price"] * qty
                     self.buying_power -= actual_price
                     ret_1 = ret.copy()
                     self.orders.remove(ret)
@@ -238,9 +269,13 @@ class PaperBroker(API):
 
             self.equity = self._calc_equity()
 
+        if "filled_time" not in ret:
+            ret["filled_time"] = None
+        if "filled_price" not in ret:
+            ret["filled_price"] = None
+
         debugger.debug(f"Returning status: {ret}")
-        stocks, cryptos = self.stocks, self.cryptos
-        debugger.debug(f"Positions:\n{stocks}\n=========\n{cryptos}")
+        debugger.debug(f"Positions:\n{self.stocks}\n=========\n{self.cryptos}")
         debugger.debug(f"Equity:{self._calc_equity()}")
         self._save_account()
         return ret
@@ -268,8 +303,8 @@ class PaperBroker(API):
                     )
                 elif ret["limit_price"] < price:
                     limit_price = ret["limit_price"]
-                    debugger.info(
-                        f"Limit price for {sym} is less than current price ({limit_price} < {price})."
+                    debugger.warn(
+                        f"Limit price for {sym} is less than current price ({limit_price} < {price}). Cannot buy {sym}!"
                     )
                 else:
                     # If asset already exists, buy more. If not, add a new entry
@@ -327,8 +362,7 @@ class PaperBroker(API):
             self.equity = self._calc_equity()
 
         debugger.debug(f"Returning status: {ret}")
-        stocks, cryptos = self.stocks, self.cryptos
-        debugger.debug(f"Positions:\n{stocks}\n{self.options}\n{cryptos}")
+        debugger.debug(f"Positions:\n{self.stocks}\n{self.options}\n{self.cryptos}")
         debugger.debug(f"Equity:{self._calc_equity()}")
         self._save_account()
         return ret
@@ -349,7 +383,10 @@ class PaperBroker(API):
         limit_price: float,
         in_force: str = "gtc",
         extended: bool = False,
-    ):
+    ) -> Dict[str, Any]:
+
+        self._validate_order(side, quantity, limit_price)
+
         data = {
             "type": "STOCK",
             "symbol": symbol,
@@ -365,6 +402,9 @@ class PaperBroker(API):
         self.orders.append(data)
         self.order_id += 1
         ret = {"order_id": data["order_id"], "symbol": data["symbol"]}
+        if side == "buy":
+            self.buying_power -= quantity * limit_price
+
         return ret
 
     def order_crypto_limit(
@@ -375,7 +415,10 @@ class PaperBroker(API):
         limit_price: float,
         in_force: str = "gtc",
         extended: bool = False,
-    ):
+    ) -> Dict[str, Any]:
+
+        self._validate_order(side, quantity, limit_price)
+
         data = {
             "type": "CRYPTO",
             "symbol": "@" + symbol,
@@ -391,6 +434,9 @@ class PaperBroker(API):
         self.orders.append(data)
         self.order_id += 1
         ret = {"order_id": data["order_id"], "symbol": data["symbol"]}
+
+        if side == "buy":
+            self.buying_power -= quantity * limit_price
         return ret
 
     def order_option_limit(
@@ -403,7 +449,9 @@ class PaperBroker(API):
         exp_date: dt.datetime,
         strike: float,
         in_force: str = "gtc",
-    ):
+    ) -> Dict[str, Any]:
+
+        self._validate_order(side, quantity, limit_price)
 
         data = {
             "type": "OPTION",
@@ -420,16 +468,20 @@ class PaperBroker(API):
 
         self.orders.append(data)
         self.order_id += 1
+        if side == "buy":
+            self.buying_power -= quantity * limit_price
+
         return {"order_id": data["order_id"], "symbol": data["symbol"]}
 
     # ------------- Helper methods ------------- #
 
-    def _calc_equity(self):
+    def _calc_equity(self) -> float:
         """
         Calculates the total worth of the broker by adding together the
         worth of all stocks, cryptos, options and cash in the broker.
         """
         e = 0
+        # Add value of current assets
         for asset in self.stocks + self.cryptos + self.options:
             add = asset["avg_price"] * asset["quantity"]
             if "multiplier" in asset:
@@ -438,7 +490,12 @@ class PaperBroker(API):
         e += self.cash
         return float(e)
 
-    def apply_commission(self, inital_price: float, commission_fee, side: str) -> float:
+    def apply_commission(
+        self,
+        inital_price: float,
+        commission_fee: Union[float, str, Dict[str, Any]],
+        side: str,
+    ) -> float:
         if side == "buy":
             f = lambda a, b: float(a + b)
         elif side == "sell":
