@@ -1,8 +1,8 @@
 # Builtins
 from logging import debug
 import re
-import threading
-import traceback
+
+# import traceback
 import sys
 from sys import exit
 from signal import signal, SIGINT
@@ -14,12 +14,21 @@ import tzlocal
 
 # Submodule imports
 from harvest.utils import *
-from harvest.storage import BaseStorage
-from harvest.api.yahoo import YahooStreamer
-from harvest.api.paper import PaperBroker
-from harvest.storage import BaseStorage
+from harvest.definitions import *
+
+# from harvest.storage import BaseStorage
+# from harvest.api._base import API
+# from harvest.api.yahoo import YahooStreamer
+# from harvest.api.paper import PaperBroker
+# from harvest.storage import BaseStorage
 from harvest.server import Server
 
+from rich.console import Console
+from rich.table import Table
+from rich.panel import Panel
+from rich import box
+from harvest.util.factory import *
+
 
 class LiveTrader:
     """
@@ -38,37 +47,64 @@ class LiveTrader:
         Interval.DAY_1,
     ]
 
-    def __init__(self, streamer=None, broker=None, storage=None, debug=False):
+    def __init__(
+        self,
+        streamer: str = None,
+        broker: str = None,
+        storage: str = "base",
+        debug: bool = False,
+    ) -> None:
         """Initializes the Trader."""
 
         self._init_checks()
         self._set_streamer_broker(streamer, broker)
-        self.storage = BaseStorage() if storage is None else storage
+        self._set_storage(storage)
         self._init_attributes()
         self._setup_debugger(debug)
+        self.console = Console()
 
-    def _init_checks(self):
+    def _init_checks(self) -> None:
         # Harvest only supports Python 3.9 or newer.
         if sys.version_info[0] < 3 or sys.version_info[1] < 9:
             raise Exception("Harvest requires Python 3.9 or above.")
 
-    def _set_streamer_broker(self, streamer, broker):
+    def _set_streamer_broker(self, streamer: str, broker: str) -> None:
         """Sets the streamer and broker."""
-        # If streamer is not specified, use YahooStreamer
-        self.streamer = YahooStreamer() if streamer is None else streamer
-        # Broker must be specified
-        if broker is None:
-            # TODO: Raise exception if specified class is streaming only
-            if isinstance(self.streamer, YahooStreamer):
-                raise Exception("Broker must be specified")
-            else:
-                self.broker = self.streamer
+        if streamer is None:
+            raise Exception("Streamer is not specified.")
+        self.streamer_str = streamer
+        if not broker:
+            self.broker_str = streamer
         else:
-            self.broker = broker
+            self.broker_str = broker
 
-    def _init_attributes(self):
+        if self.streamer_str not in apis:
+            raise Exception(f"Streamer {self.streamer_str} is not recognized.")
+        if self.broker_str not in apis:
+            raise Exception(f"Broker {self.broker_str} is not recognized.")
 
-        signal(SIGINT, self.exit)
+        if self.streamer_str not in streamers:
+            raise Exception(f"{self.streamer_str} cannot be used as a streamer.")
+        if self.broker_str not in brokers:
+            raise Exception(f"{self.broker_str} cannot be used as a broker.")
+
+    def _set_storage(self, storage: str) -> None:
+        """Sets the storage."""
+        if storage is None:
+            raise Exception("Storage is not specified.")
+        self.storage_str = storage
+
+    def _init_attributes(self) -> None:
+
+        try:
+            signal(SIGINT, self.exit)
+        except:
+            debugger.warning(
+                "Can't use signal, Harvest is running in a non-main thread."
+            )
+
+        self.start_streamer = True
+        self.skip_init = False
 
         self.watchlist = []  # List of securities specified in this class
         self.algo = []  # List of algorithms to run.
@@ -77,8 +113,6 @@ class LiveTrader:
 
         self.stats = Stats(None, tzlocal.get_localzone(), None)
 
-        self.storage.setup(self.stats)
-
         self.func = Functions(
             self.buy,
             self.sell,
@@ -86,32 +120,58 @@ class LiveTrader:
             self.fetch_chain_info,
             self.fetch_option_market_data,
             self.get_asset_quantity,
-            self.storage.load,
-            self.storage.store,
-            self.storage.load_daytrade,
+            self.load,
+            self.store,
+            self.load_daytrade,
         )
 
         self.account = Account()
         self.positions = self.account.positions
         self.orders = self.account.orders
 
-    def _setup_debugger(self, debug):
+    def _setup_debugger(self, debug: bool) -> None:
         # Set up logger
         if debug:
             debugger.setLevel("DEBUG")
 
-        debugger.debug(
-            f"Streamer: {type(self.streamer).__name__}\nBroker: {type(self.broker).__name__}\nStorage: {type(self.storage).__name__}"
+        # debugger.debug(
+        #     f"Streamer: {type(self.streamer).__name__}\nBroker: {type(self.broker).__name__}\nStorage: {type(self.storage).__name__}"
+        # )
+
+    def _init_param_streamer_broker(self, interval, aggregations) -> None:
+        # Initialize a dict of symbols and the intervals they need to run at
+        self._setup_params(self.watchlist, interval, aggregations)
+        self.watchlist = list(self.stats.watchlist_cfg.keys())
+        if not self.watchlist:
+            raise Exception("No securities were added to watchlist")
+
+        # Load and set up streamer and broker
+        self.broker = load_api(self.broker_str)()
+        self.broker.setup(self.stats, self.account, self.main)
+        self.console.print(
+            f"- [cyan]{self.broker.__class__.__name__}[/cyan] setup complete"
         )
+        if self.broker_str != self.streamer_str:
+            self.streamer = load_api(self.streamer_str)()
+            self.streamer.setup(self.stats, self.account, self.main)
+            self.broker.streamer = self.streamer
+            self.streamer.broker = self.broker
+            self.console.print(
+                f"- [cyan]{self.streamer.__class__.__name__}[/cyan] setup complete"
+            )
+        else:
+            self.streamer = self.broker
+        self.storage = load_storage(self.storage_str)()
+        self.storage.setup(self.stats)
 
     def start(
         self,
-        interval="5MIN",
-        aggregations=[],
-        sync=True,
-        server=False,
-        all_history=True,
-    ):
+        interval: str = "5MIN",
+        aggregations: List = [],
+        sync: bool = True,
+        server: bool = False,
+        all_history: bool = True,
+    ) -> None:
         """Entry point to start the system.
 
         :param str? interval: The interval to run the algorithm. defaults to '5MIN'
@@ -121,54 +181,148 @@ class LiveTrader:
         :param bool? sync: If true, the system will sync with the broker and fetch current positions and pending orders. defaults to true.
         :param bool? all_history: If true, gets all history for all the given assets and if false only get data in the past three days.
         """
-        debugger.debug("Setting up Harvest...")
+        debugger.debug("Setting up Harvest")
+        with self.console.status(
+            "[bold green] Setting up Trader...[/bold green]"
+        ) as status:
+            if not self.skip_init:
+                self._init_param_streamer_broker(interval, aggregations)
+            # If sync is on, call the broker to load pending orders and all positions currently held.
+            if sync:
+                self._setup_stats()
+                for s in self.positions.stock_crypto:
+                    self.watchlist.append(s.symbol)
+                for s in self.positions.option:
+                    self.watchlist.append(s.base_symbol)
+                self.watchlist.extend(self.orders.stock_crypto_symbols)
+                self.console.print(f"- Finished syncing data with broker")
 
-        # If sync is on, call the broker to load pending orders and all positions currently held.
-        if sync:
-            self._setup_stats()
-            for s in self.positions.stock_crypto:
-                self.watchlist.append(s.symbol)
-            for s in self.positions.option:
-                self.watchlist.append(s.base_symbol)
-            self.watchlist.extend(self.orders.stock_crypto_symbols)
+            # Initialize the account
+            self._setup_account()
+            self.storage.init_performace_data(self.account.equity, self.stats.timestamp)
 
-        # Remove duplicates in watchlist
-        self.watchlist = list(set(self.watchlist))
-        debugger.debug(f"Watchlist: {self.watchlist}")
+            # Initialize the storage
+            self._storage_init(all_history)
+            self.console.print(
+                f"- [cyan]{self.storage.__class__.__name__}[/cyan] setup complete"
+            )
 
-        # Initialize a dict of symbols and the intervals they need to run at
-        self._setup_params(self.watchlist, interval, aggregations)
-        self.watchlist = self.stats.watchlist_cfg.keys()
+            for a in self.algo:
+                a.init(self.stats, self.func, self.account)
+                a.trader = self
+                a.setup()
+            self.console.print("- All algorithms initialized")
 
-        if not self.watchlist:
-            raise Exception("No securities were added to watchlist")
+        self.console.print("> [bold green]Trader initialization complete[/bold green]")
 
-        self.broker.setup(self.stats, self.account, self.main)
-        if self.broker != self.streamer:
-            # Only call the streamer setup if it is a different
-            # instance than the broker otherwise some brokers can fail!
-            self.streamer.setup(self.stats, self.account, self.main)
-
-        # Initialize the account
-        self._setup_account()
-        self.storage.init_performace_data(self.account.equity, self.stats.timestamp)
-
-        # Initialize the storage
-        self._storage_init(all_history)
-
-        for a in self.algo:
-            a.init(self.stats, self.func, self.account)
-            a.trader = self
-            a.setup()
-
-        debugger.debug("Setup complete")
+        self._print_status()
 
         if server:
             self.server.start()
 
-        self.streamer.start()
+        if self.start_streamer:
+            self.streamer.start()
 
-    def _setup_stats(self):
+    def _print_account(self) -> None:
+        a = self.account
+
+        def p_line(k, v):
+            return f"{k}", f"[bold white]{v}[/bold white]"
+
+        table = Table(
+            title=a.account_name,
+            show_header=False,
+            show_lines=True,
+            box=box.ROUNDED,
+        )
+        table.add_row(*p_line("Cash ($)", a.cash))
+        table.add_row(*p_line("Equity ($)", a.equity))
+        table.add_row(*p_line("Buying Power ($)", a.buying_power))
+
+        self.console.print(table)
+
+    def _print_positions(self) -> None:
+        def bold(s):
+            return f"[bold white]{s}[/bold white]"
+
+        def red_or_green(s):
+            if s >= 0:
+                return f"[bold green]{s}[/bold green]"
+            else:
+                return f"[bold red]{s}[/bold red]"
+
+        def print_table(title, positions):
+
+            if len(positions) == 0:
+                return
+
+            stock_table = Table(
+                title=title,
+                show_lines=True,
+                box=box.ROUNDED,
+            )
+            stock_table.add_column("Symbol")
+            stock_table.add_column("Quantity")
+            stock_table.add_column("Current Price")
+            stock_table.add_column("Avg. Cost")
+            stock_table.add_column("Profit/Loss ($)")
+            stock_table.add_column("Profit/Loss (%)")
+
+            for p in positions:
+                ret_prefix = "+" if p.profit >= 0 else ""
+                per_prefix = "🚀 " if p.profit_percent >= 0.1 else ""
+                stock_table.add_row(
+                    f"{p.symbol}",
+                    f"{p.quantity}",
+                    f"${p.current_price}",
+                    f"${p.avg_cost}",
+                    f"{per_prefix} {ret_prefix} {red_or_green(p.profit)}",
+                    f"{per_prefix} {ret_prefix} {red_or_green(p.profit_percent)}",
+                )
+            self.console.print(stock_table)
+
+        print_table("Stock Positions", self.positions.stock)
+        print_table("Crypto Positions", self.positions.crypto)
+
+        if len(self.positions.option) == 0:
+            return
+
+        option_table = Table(
+            title="Option Positions",
+            show_lines=True,
+            box=box.ROUNDED,
+        )
+        option_table.add_column("Symbol")
+        option_table.add_column("Strike Price")
+        option_table.add_column("Expiration Date")
+        option_table.add_column("Type")
+        option_table.add_column("Quantity")
+        option_table.add_column("Current Price")
+        option_table.add_column("Avg. Cost")
+        option_table.add_column("Profit/Loss ($)")
+        option_table.add_column("Profit/Loss (%)")
+
+        for p in self.positions.option:
+            ret_prefix = "+" if p.profit >= 0 else ""
+            per_prefix = "🚀 " if p.profit_percent >= 0.1 else ""
+            option_table.add_row(
+                f"{p.base_symbol}",
+                f"{p.strike}",
+                f"{p.expiration}",
+                f"{p.option_type}",
+                f"{p.quantity}",
+                f"${p.current_price}",
+                f"${p.avg_cost}",
+                f"{per_prefix} {ret_prefix} {red_or_green(p.profit)}",
+                f"{per_prefix} {ret_prefix} {red_or_green(p.profit_percent)}",
+            )
+        self.console.print(option_table)
+
+    def _print_status(self) -> None:
+        self._print_account()
+        self._print_positions()
+
+    def _setup_stats(self) -> None:
         """Initializes local cache of stocks, options, and crypto positions."""
 
         # Get any pending orders
@@ -178,7 +332,9 @@ class LiveTrader:
 
         self._fetch_account_data()
 
-    def _setup_params(self, watchlist, interval, aggregations):
+    def _setup_params(
+        self, watchlist: List[str], interval: str, aggregations: List[str]
+    ) -> None:
         """
         Sets up configuration parameters for the Trader, notably
         the 'interval' attribute.
@@ -189,6 +345,8 @@ class LiveTrader:
             sym: {"interval": interval, "aggregations": aggregations}
             for sym in watchlist
         }
+
+        debugger.debug(f"Watchlist cfg: {watchlist_cfg_tmp}")
         # Update the dict based on parameters specified in Algo class
         for a in self.algo:
             a.config()
@@ -227,6 +385,8 @@ class LiveTrader:
                     if agg not in watchlist_cfg_tmp[sym]["aggregations"]:
                         watchlist_cfg_tmp[sym]["aggregations"].append(agg)
 
+        debugger.debug(f"Watchlist config: {watchlist_cfg_tmp}")
+
         # Remove any duplicates in the dict
         for sym in watchlist_cfg_tmp:
             new_agg = list((set(watchlist_cfg_tmp[sym]["aggregations"])))
@@ -234,7 +394,7 @@ class LiveTrader:
 
         self.stats.watchlist_cfg = watchlist_cfg_tmp
 
-    def _setup_account(self):
+    def _setup_account(self) -> None:
         """Initializes local cache of account info.
         For testing, it should manually be specified
         """
@@ -243,7 +403,7 @@ class LiveTrader:
             raise Exception("Failed to load account info from broker.")
         self.account.init(ret)
 
-    def _storage_init(self, all_history: bool):
+    def _storage_init(self, all_history: bool) -> None:
         """
         Initializes the storage.
         :all_history: bool :
@@ -259,7 +419,7 @@ class LiveTrader:
 
     # ================== Functions for main routine =====================
 
-    def main(self, df_dict):
+    def main(self, df_dict: Dict[str, pd.DataFrame]) -> None:
         """
         Main loop of the Trader.
         """
@@ -293,13 +453,15 @@ class LiveTrader:
 
         self._update_local_cache(df_dict)
 
+        self._print_positions()
+
         new_algo = []
         for a in self.algo:
             if not is_freq(self.stats.timestamp, a.interval):
                 new_algo.append(a)
                 continue
             try:
-                debugger.info(f"Running algo: {a}")
+                # debugger.info(f"Running algo: {a}")
                 a.main()
                 new_algo.append(a)
             except Exception as e:
@@ -307,7 +469,7 @@ class LiveTrader:
                     f"Algorithm {a} failed, removing from algorithm list.\n"
                 )
                 debugger.warning(f"Exception: {e}\n")
-                debugger.warning(f"Traceback: {traceback.format_exc()}\n")
+                # debugger.warning(f"Traceback: {traceback.format_exc()}\n")
 
         if len(new_algo) <= 0:
             debugger.critical("No algorithms to run")
@@ -318,7 +480,7 @@ class LiveTrader:
         self.broker.exit()
         self.streamer.exit()
 
-    def _update_order_queue(self):
+    def _update_order_queue(self) -> bool:
         """Check to see if outstanding orders have been accepted or rejected
         and update the order queue accordingly.
         """
@@ -334,8 +496,6 @@ class LiveTrader:
             debugger.debug(f"Updating status of order {order.order_id}")
             order.update(stat)
 
-        debugger.debug(f"Updated order queue: {self.orders}")
-
         order_filled = False
         for order in self.orders.orders:
             # TODO: handle cancelled orders
@@ -353,11 +513,12 @@ class LiveTrader:
                     order.filled_price,
                 )
         self.orders.remove_non_open()
+        debugger.debug(f"Updated order queue: {self.orders}")
 
         # if an order was processed, update the positions and account info
         return order_filled
 
-    def _update_local_cache(self, df_dict):
+    def _update_local_cache(self, df_dict: Dict[str, pd.DataFrame]) -> None:
         """Update local cache of stocks, options, and crypto positions"""
         # Update entries in local cache
         # API should also be called if load_watch is false, as there is a high chance
@@ -369,7 +530,7 @@ class LiveTrader:
         for p in self.positions.stock_crypto:
             symbol = p.symbol
             if symbol in df_dict:
-                price = df_dict[symbol][symbol]["close"]
+                price = df_dict[symbol].iloc[-1][symbol]["close"]
             elif (
                 symbol not in self.watchlist
             ):  # handle cases when user has an asset not in watchlist
@@ -386,7 +547,8 @@ class LiveTrader:
 
         debugger.debug(f"Updated positions: {self.positions}")
 
-    def _fetch_account_data(self):
+    def _fetch_account_data(self) -> None:
+        debugger.debug(f"Fetching account data")
         stock_pos = [
             Position(p["symbol"], p["quantity"], p["avg_price"])
             for p in self.broker.fetch_stock_positions()
@@ -410,6 +572,7 @@ class LiveTrader:
         self.positions.update(stock_pos, option_pos, crypto_pos)
 
         ret = self.broker.fetch_account()
+
         self.account.init(ret)
 
     # --------------------- Interface Functions -----------------------
@@ -423,6 +586,15 @@ class LiveTrader:
     def fetch_option_market_data(self, *args, **kwargs):
         return self.streamer.fetch_option_market_data(*args, **kwargs)
 
+    def load(self, *args, **kwargs):
+        return self.storage.load(*args, **kwargs)
+
+    def store(self, *args, **kwargs):
+        return self.storage.store(*args, **kwargs)
+
+    def load_daytrade(self, *args, **kwargs):
+        return self.storage.load_daytrade(*args, **kwargs)
+
     def buy(self, symbol: str, quantity: int, in_force: str, extended: bool):
         # Check if user has enough buying power
         buy_power = self.account.buying_power
@@ -448,6 +620,7 @@ class LiveTrader:
             )
             return None
         ret = self.broker.buy(symbol, quantity, limit_price, in_force, extended)
+        debugger.debug(f"Account info after buy: {self.account}")
 
         if ret is None:
             debugger.debug("BUY failed")
@@ -460,6 +633,9 @@ class LiveTrader:
     def sell(self, symbol: str, quantity: int, in_force: str, extended: bool):
         # Check how many of the given asset we currently own
         owned_qty = self.get_asset_quantity(symbol, True, False)
+        if owned_qty == 0:
+            debugger.error(f"You do not own any {symbol}")
+            return None
         if quantity > owned_qty:
             debugger.debug(
                 "SELL failed: More quantities are being sold than currently owned."
@@ -485,7 +661,7 @@ class LiveTrader:
 
     # ================ Helper Functions ======================
     def get_asset_quantity(
-        self, symbol, include_pending_buy, include_pending_sell
+        self, symbol: str, include_pending_buy: bool, include_pending_sell: bool
     ) -> float:
         """Returns the quantity owned of a specified asset.
 
@@ -511,21 +687,21 @@ class LiveTrader:
 
         if include_pending_buy:
             owned_qty += sum(
-                o["quantity"]
+                o.quantity
                 for o in self.orders.orders
-                if o["symbol"] == symbol and o["side"] == "buy"
+                if o.symbol == symbol and o.side == "buy"
             )
 
         if not include_pending_sell:
             owned_qty -= sum(
-                o["quantity"]
+                o.quantity
                 for o in self.orders.orders
-                if o["symbol"] == symbol and o["side"] == "sell"
+                if o.symbol == symbol and o.side == "sell"
             )
 
         return owned_qty
 
-    def set_algo(self, algo):
+    def set_algo(self, algo) -> None:
         """Specifies the algorithm to use.
 
         :param Algo algo: The algorithm to use. You can either pass in a single Algo class, or a
@@ -533,10 +709,10 @@ class LiveTrader:
         """
         self.algo = algo if isinstance(algo, list) else [algo]
 
-    def add_algo(self, algo):
+    def add_algo(self, algo) -> None:
         self.algo.append(algo)
 
-    def set_symbol(self, symbol):
+    def set_symbol(self, symbol: Union[List[str], str]) -> None:
         """Specifies the symbol(s) to watch.
 
         Cryptocurrencies should be prepended with an `@` to differentiate them from stocks.
@@ -549,7 +725,7 @@ class LiveTrader:
         """
         self.watchlist = symbol if isinstance(symbol, list) else [symbol]
 
-    def day_trade_count(self):
+    def day_trade_count(self) -> None:
         # Get the 5-day trading window
         calendar = self.storage.load_calendar()
         open_days = calendar.loc[self.calendar["is_open"] == True]
@@ -577,18 +753,18 @@ class PaperTrader(LiveTrader):
     A class for trading in the paper trading environment.
     """
 
-    def __init__(self, streamer=None, storage=None, debug=False):
+    def __init__(
+        self, streamer: str = None, storage: str = None, debug: bool = False
+    ) -> None:
         """Initializes the Trader."""
 
         self._init_checks()
 
         # If streamer is not specified, use YahooStreamer
-        self.streamer = YahooStreamer() if streamer is None else streamer
-        self.broker = PaperBroker(streamer=self.streamer)
+        self.streamer_str = "yahoo" if streamer is None else streamer
+        self.broker_str = "paper"
+        self.storage_str = "base" if storage is None else storage
 
-        self.storage = (
-            BaseStorage() if storage is None else storage
-        )  # Initialize the storage
         self._init_attributes()
-
         self._setup_debugger(debug)
+        self.console = Console()
